# Copyright INRA/CNRS

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

	Utils.pl

=head1 DESCRIPTION

	Set of utility functions

=head1 AUTHORS

	Jerome Gouzy      - Jerome.Gouzy@toulouse.inra.fr
	Sebastien Carrere - Sebastien.Carrere@toulouse.inra.fr

=cut

=head3 function System

 Title        : System
 Usage        : System($cde,"stderr,log,errcode")
 Prerequisite : none
 Function     : Run a system command, can log the command on the LOG/STDERR streams or die if the command returns an error code
 Returns      : none
 Args         : $cde
 		$how: a string that can contain "stderr,log,errcode"
 Globals      : LOG/STDERR streams

=cut

sub System
{
    my ($cde, $how) = @_;

    print STDERR "$cde\n" if ($how =~ /stderr/i);
    print LOG "$cde\n"    if ($how =~ /log/i);
    print STDOUT "$cde\n" if ($how =~ /stdout/i);
    
    my $ret = system("$cde");
    if ($how =~ /errcode/)
    {
        die "ERROR, the command >$cde< has returned an error code: $ret\n" if ($ret);
    }
}

=head3 function GetFluxFromFilename

 Title        : GetFluxFromFilename
 Usage        : GetFluxFromFilename($filename)
 Prerequisite : none
 Function     : Depending on filename extension (.gz,.bz2,.Z), build and return a string that can be opened as a stream
 Returns      : $filename
 Args         : $filename
 Globals      : none

=cut

sub GetFluxFromFilename
{
    my ($filename) = @_;
    my ($ext) = $filename =~ /\.(\w+)$/;

    my %h_ext = ('Z' => 'zcat', 'gz' => 'gzip -cd', 'bz2' => 'bzip2 -cd');

    return "$h_ext{$ext} $filename | " if (defined($h_ext{$ext}));
    return $filename;
}

=head3 function GetDate

 Title        : GetDate
 Usage        : GetDate()
 Prerequisite : none
 Function     : return the date in the format YYYYMMDD
 Returns      : $date
 Args         : none
 Globals      : none

=cut

sub GetDate
{
    my $date = `date +"20%y%m%d"`;
    chomp $date;
    return ($date);
}

=head3 function IsEmailValid

 Title        : IsEmailValid
 Usage        : IsEmailValid($mail)
 Prerequisite : none
 Function     : check validity of an email address (syntax)
 Returns      : boolean
 Args         : none
 Globals      : none

=cut

sub IsEmailValid
{
    my $mail = shift();

    if ($mail =~ /\S+\@\S+\.\S+/)
    {
        return 1;
    }
    else
    {
        return 0;
    }

}

sub SendMail
{
    my ($message, $email, $subject) = @_;

    use ParamParser;
    my $o_param;
    $o_param = &iANT::Cfg::Load($o_param);

    my $mail_prog = $o_param->Get('unix_mail');

     $subject = "iANT2 // Analyses Message" if (! defined $subject);
     
    my $mail_cmd = "$mail_prog -s \"$subject\" $email";

    open(MAIL, "| $mail_cmd") || die("ERROR $mail_cmd : $!");
    print MAIL <<END;
$message
END
    close MAIL;

}


=head3 function GetRandomWord

 Title        : GetRandomWord
 Usage        : GetRandomWord($size)
 Prerequisite : none
 Function     : returns a random word (only letters and numbers)
 Returns      : a word
 Args         : $size : size of the word
 Globals      : none

=cut


sub GetRandomWord
{
	my $size = shift;
	
	$size = 8 if (!defined $size);
	
	my @a_letters = ('A' .. 'Z', 'a' .. 'z', 0 .. 9);

	my $word = '';
	for (my $i = 1; $i <= $size; $i++)
	{
		$word .= $a_letters[int(rand ($#a_letters + 1))];
	}
	return $word;
}


sub GetRawFeaturesNumber
{
    my $file = shift();
    my $bdbh = shift();
    use ParamParser;
    my $o_param;
    $o_param = &iANT::Cfg::Load($o_param);
    my $bdbh_binary = $o_param->Get('bdbh');
    my $unix_grep = $o_param->Get('unix_grep');
    my $cmd = "$bdbh_binary -d $bdbh cat $file | $unix_grep -c 'feature '";
    my $number = `$cmd`;
    chomp $number;
    return $number;
    


}

=pod

=head1 IN iANT::Common now (2010/06/30)

=head3 function SplitDBFeatureXMLIntoFeaturesXML

 Title        : SplitDBFeatureXMLIntoFeaturesXML
 Usage        : SplitDBFeatureXMLIntoFeaturesXML($db_feature_xml_file)
 Function     : returns an array ref of atomic feature xml data
 Returns      : array ref
 Args         : $db_feature_xml_file : iANT::DB::Feature XML file
 Globals      : none

=cut

=pod
sub SplitDBFeatureXMLIntoFeaturesXML
{
    my $file = shift;
    
    my @a_xml_features = ();
    if (-e $file)
    {
        my $cmd = 'sed -i \'s/feature><feature/feature>\n<feature/g\'';
        system("$cmd $file");
        my $fh_in = new IO::File($file);

        my $feature_xml = '';
        my $in_feature = 0;
        while (my $line = <$fh_in>)
        {
            chomp $line;
            if ($line =~ /<feature /)
            {
                $feature_xml = "$line\n";
                $in_feature = 1;
            }
            elsif ($in_feature == 1)
            {
                $feature_xml .= "$line\n";
            }
            if ($line =~ /<\/feature>/)
            {
                push(@a_xml_features,$feature_xml);

            }
        }
        $fh_in->close;
    
    
    
    }
    else
    {
        $file =~ s/feature><feature/feature>\n<feature/g;
        
        my $feature_xml = '';
        my $in_feature = 0;
        my @a_lines = split ("\n",$file);
        chomp @a_lines;
        foreach my $line (@a_lines)
        {
            if ($line =~ /<feature /)
            {
                $feature_xml = "$line\n";
                $in_feature = 1;
            }
            elsif ($in_feature == 1)
            {
                $feature_xml .= "$line\n";
            }
            if ($line =~ /<\/feature>/)
            {
                push(@a_xml_features,$feature_xml);

            }
        }
    }
    
    
    
    return \@a_xml_features;
}

=cut

=pod

=head3 function SplitDBEntriesXMLIntoEntriesXML

 Title        : SplitDBEntriesXMLIntoEntriesXML
 Usage        : SplitDBEntriesXMLIntoEntriesXML($db_entry_xml_file)
 Function     : returns an array ref of atomic entry xml data
 Returns      : array ref
 Args         : $db_feature_xml_file : iANT::DB::Entry XML file
 Globals      : none

=cut

=pod
sub SplitDBEntriesXMLIntoEntriesXML
{
    my $file = shift;
    
    my @a_xml_entries = ();
    my $cmd = 'sed -i \'s/entry><entry/entry>\n<entry/g\'';
    system("$cmd $file");
    
    my $fh_in = new IO::File($file);
    
    my $entry_xml = '';
    my $in_entry = 0;
    while (my $line = <$fh_in>)
    {
        chomp $line;
        if ($line =~ /<entry /)
        {
            $entry_xml = "$line\n";
            $in_entry = 1;
        }
        elsif ($in_entry == 1)
        {
            $entry_xml .= "$line\n";
        }
        if ($line =~ /<\/entry>/)
        {
            push(@a_xml_entries,$entry_xml);
            
        }
    }
    $fh_in->close;
    
    return \@a_xml_entries;
}
=cut

sub GetPubmedFromSPTRAccession
{
    my $file = shift();
    my $accession = shift();
    my $renderer = shift();
    
    $renderer = 'text' if (! defined $renderer);
    
    use ParamParser;
    my $o_param;
    $o_param = &iANT::Cfg::Load($o_param);
    my $unix_grep = $o_param->Get('unix_grep');
    
    my $cmd = "$unix_grep -w $accession $file";

    my $pmids_line = `$cmd`;
    chomp $pmids_line;
    
    my @a_pubmed_id = ();
    if ($pmids_line =~ /\s+\d+/)
    {
        while ($pmids_line =~ /\s+(\d+)/g)
        {
            push (@a_pubmed_id,$1);
        }
    }

    if ($#a_pubmed_id >= 0)
    {
    if ($renderer =~ /html/i)
    {
        my $content = 'Pubmed ';
        foreach my $id (@a_pubmed_id)
        {
            $content .= '<a href="http://www.ncbi.nlm.nih.gov/sites/entrez?holding=&db=pubmed&cmd=search&term='. $id . '" target="pubmed">' . $id .'</a>,' if ($id =~ /^\d+$/);
        }
        return $content;
   }
   elsif ($renderer =~ /text/i)
   {
       my $content = join (' ' , @a_pubmed_id);
       return $content;
   }
   }
}


#
#
# ############ Sequence related utilities (maybe a new library ?) #########
#
#

=pod

=item function GetAllowedMethods

 Title        : GetAllowedMethods
 Usage        : GetAllowedMethods
 Function     : List all allowed methods that could be applied on an entry
 Returns      : \%h_allowed_methods
 Args         : none
 Globals      : none

=cut

sub GetAllowedMethods
{
    my %h_allowed_methods = (
                  'seg'            => {'program' => 'seg'},
                  'tmpred'         => {'program' => 'tmpred'},
                  'predotar'       => {'program' => 'predotar'},
                  'hamap'          => {'program' => 'pfscan', 'dbname' => 'hamap'},
                  'genomethreader' => {'program' => 'gth'},
                  'prosite'        => {'program' => 'pfscan', 'dbname' => 'prosite'},
                  'blastpsptr' =>
                    {'program' => 'ncbi-blastp', 'dbname' => 'sptr', 'blast_parameters' => '"-e 1e-6 "', 'trust' => ''},
                  'blastpprodom' => {
                                     'program'          => 'ncbi-blastp',
                                     'dbname'           => 'prodom',
                                     'trust'            => '',
                                     'blast_parameters' => '"-e 1e-6 "',
                                     'trust'            => ''
                    },
                  'blastpspecific'      => {'program' => 'ncbi-blastp', 'dbname' => 'organism_specific_proteic_db'},
                  'ncbi-blastnspecific' => {
                                            'program'          => 'ncbi-blastn',
                                            'dbname'           => 'organism_specific_nucleic_db',
                                            'blast_parameters' => '" "',
                                            'trust'            => ''
                    },
                  'wu-blastnspecific' => {
                                          'program'          => 'wu-blastn',
                                          'dbname'           => 'organism_specific_nucleic_db',
                                          'blast_parameters' => '" "',
                                          'trust'            => ''
                    }
                    );

    return \%h_allowed_methods;
}

=pod

=item function GetAllowedMethodsNames

 Title        : GetAllowedMethodsNames
 Usage        : GetAllowedMethodsNames
 Function     : List all keys of hash return by GetAllowedMethods
 Returns      : \%h_allowed_methods
 Args         : none
 Globals      : none

=cut

sub GetAllowedMethodsNames
{
    my $rh_allowed_methods      = &GetAllowedMethods();
    my @a_allowed_methods_names = keys %{$rh_allowed_methods};

    return \@a_allowed_methods_names;
}

=pod

=item function GetDefaultMethodsNames

 Title        : GetDefaultMethodsNames
 Usage        : GetDefaultMethodsNames($type)
 Function     : List all keys of hash return by GetAllowedMethods for a specific type
			 according to configuration file settings
 Returns      : \%h_allowed_methods
 Args         : $type nucleic|proteic
 Globals      : [conf] mgt_analyses_default_methods_nucleic, mgt_analyses_default_methods_proteic

=cut

sub GetDefaultMethodsNames
{
    my $type = shift;
    use ParamParser;
    my $o_param;
    $o_param = &iANT::Cfg::Load($o_param);

    my $default_methods_string = $o_param->Get('mgt_analyses_default_methods_' . $type);

    my @a_default_methods = split(/,/, $default_methods_string);

    return \@a_default_methods;

}

=pod

=item function GetFrame

 Title        : GetFrame
 Usage        : GetFrame($begin,$end,$strand,$len)
 Prerequisite : $begin < end
 Function     : return the 'blast' frame
 Returns      : $frame ('+1','+2','+3','-1','-2','-3')
 Args         : $begin,$end,$strand,$len
 Globals      : none

=cut

sub GetFrame
{
    my ($begin, $end, $strand, $len) = @_;
    my $frame;

    if ($strand eq '+')
    {
        $frame = "+1" if (($begin - 1) % 3 == 0);
        $frame = "+2" if (($begin - 2) % 3 == 0);
        $frame = "+3" if (($begin - 3) % 3 == 0);
    }
    else
    {
        $frame = "-1" if ((1 + $len - $end - 1) % 3 == 0);
        $frame = "-2" if ((1 + $len - $end - 2) % 3 == 0);
        $frame = "-3" if ((1 + $len - $end - 3) % 3 == 0);
    }
    return $frame;
}

=pod

=item function GetiANTUrl

 Title        : GetiANTUrl
 Usage        : GetiANTUrl(@a_url_parameters)
 Function     : build iANT main script Url
 Returns      : $url
 Args         : @a_url_parameters HTTP Get parameters
 Globals      : [conf] web_server, http_root, cgi_iant

=cut

sub GetiANTUrl
{
    my (@a_url_parameters) = (@_);
    use ParamParser;
    my $o_param;
    $o_param = &iANT::Cfg::Load($o_param);

    my $url = $o_param->Get('web_server') . $o_param->Get('http_root') . $o_param->Get('cgi_iant');

    my $param_string = join('&', @a_url_parameters);

    $param_string =~ s/^\?//;

    $url .= '?' . $param_string;

    return $url;
}

=pod

=item function GetMgtAnalysesUrl

 Title        : GetMgtAnalysesUrl
 Usage        : GetMgtAnalysesUrl(@a_url_parameters)
 Function     : build iANT management script Url
 Returns      : $url
 Args         : @a_url_parameters HTTP Get parameters
 Globals      : [conf] web_server, http_root, cgi_mgt_analyses

=cut

sub GetMgtAnalysesUrl
{
    my (@a_url_parameters) = (@_);
    use ParamParser;
    my $o_param;
    $o_param = &iANT::Cfg::Load($o_param);

    my $url = $o_param->Get('web_server') . $o_param->Get('http_root') . $o_param->Get('cgi_mgt_analyses');

    my $param_string = join('&', @a_url_parameters);

    $param_string =~ s/^\?//;

    $url .= '?' . $param_string;

    return $url;
}

=pod

=item function GetWebImgUrl

 Title        : GetWebImgUrl
 Usage        : GetWebImgUrl($img_name,$img_ext)
 Function     : build image url (this image should be in /web/img/ directory)
 Returns      : $img_url
 Args         : $img_name name of the image
			 $img_ext file extension
 Globals      : [conf] web_server, http_root

=cut

sub GetWebImgUrl
{
    my ($img_name, $img_ext) = (@_);

    use ParamParser;
    my $o_param;
    $o_param = &iANT::Cfg::Load($o_param);
    my $img_url = $o_param->Get('web_server') . $o_param->Get('http_root') . '/web/img/' . $img_name . '.' . $img_ext;
    return $img_url;
}

=pod

=item function GetWebJsUrl

 Title        : GetWebJsUrl
 Usage        : GetWebJsUrl($javascript_file_name)
 Function     : build javascript url (this script should be in /web/js/ directory)
 Returns      : $javascript_file_name
 Args         : $javascript_file_name name of the script
 Globals      : [conf] web_server, http_root

=cut

sub GetWebJsUrl
{
    my ($javascript_file_name, $img_ext) = (@_);

    use ParamParser;
    my $o_param;
    $o_param = &iANT::Cfg::Load($o_param);
    my $javascript_file_url = $o_param->Get('web_server') . $o_param->Get('http_root') . '/web/js/' . $javascript_file_name . '.js';
    return $javascript_file_url;
}

=pod

=item function GetWebPngUrl

 Title        : GetWebPngUrl
 Usage        : GetWebPngUrl($img_name)
 Function     : build PNG image url (this image should be in /web/img/ directory)
 Returns      : $img_url
 Args         : $img_name name of the image
 Globals      : none

=cut

sub GetWebPngUrl
{
    my $img_name = shift();
    my $png_url = &GetWebImgUrl($img_name, 'png');
    return $png_url;
}

=pod

=item function GetWebJpgUrl

 Title        : GetWebJpgUrl
 Usage        : GetWebJpgUrl($img_name)
 Function     : build JPG image url (this image should be in /web/img/ directory)
 Returns      : $img_url
 Args         : $img_name name of the image
 Globals      : none

=cut

sub GetWebJpgUrl
{
    my $img_name = shift();
    my $png_url = &GetWebImgUrl($img_name, 'jpg');
    return $png_url;
}

=pod

=item function GetPopUpCode

 Title        : GetPopUpCode
 Usage        : GetPopUpCode($content)
 Function     : build JavaScript Popup code
 Returns      : $popup_code
 Args         : $content HTML content of the popup 
 Globals      : none

=cut

sub GetPopUpCode
{
    my $content    = shift();
    my $popup_code = "onmouseover=\"return overlib('$content',  STICKY, MOUSEOFF)\" onmouseout=\"return nd();\"";
    return $popup_code;
}

=item function GetPopUpWithImageCode

 Title        : GetPopUpWithImageCode
 Usage        : GetPopUpWithImageCode
 Function     : build JavaScript Popup code containing an image 
 Returns      : $popup_code
 Args         : $content HTML content of the popup 
 Globals      : none

=cut

sub GetPopUpWithImageCode
{
    my $content    = shift();
    my $popup_code = "onmouseover=\"return overlib('&lt;img src=$content alt=Load_Failed />',  STICKY, MOUSEOFF, FGCOLOR, \'white\', BGCOLOR, \'white\', BORDER, 0)\" onmouseout=\"return nd();\"";
    return $popup_code;
}

=pod

=item function GetPrivilegeName

 Title        : GetPrivilegeName
 Usage        : GetPrivilegeName($level)
 Function     : returns a privilege name according to a privilege level
 Returns      : privilege name
 Args         : $level 
 Globals      : [conf] auth_privilege_$level

=cut

sub GetPrivilegeName
{
    my $level = shift();
    use ParamParser;
    my $o_param;
    $o_param = &iANT::Cfg::Load($o_param);

    if ($o_param->IsDefined('auth_privilege_' . $level))
    {
        return $o_param->Get('auth_privilege_' . $level);
    }
    else
    {
        return "unknown privilege";
    }

}

=pod

=item function GetNeighbours

 Title        : GetNeighbours
 Usage        : GetNeighbours($o_entry,$o_project)
 Function     : returns side neighbours of an entry
 Returns      : $rh_neighbours hash of previous and next iANT::Entry::Dsplay neighbours objects 
 Args         : $o_entry iANT::Entry object
			 $o_project iANT::Project object
 Globals      : none

=cut

sub GetNeighbours
{
    my ($o_entry, $o_project) = (@_);

    use ParamParser;
    my $o_param;
    $o_param = &iANT::Cfg::Load($o_param);

    #chercher d'abord dans entry si c'est pas deja def
    my $rh_neighbours;
    $rh_neighbours = &GetNeighboursFromEntry($o_entry, $o_project, $o_param);
    if ($rh_neighbours)
    {
        return $rh_neighbours;
    }
    else
    {
        $rh_neighbours = &GetNeighboursFromCoordinates($o_entry, $o_project, $o_param);
        return $rh_neighbours;
    }

    return;

}

=pod

=item function GetNeighboursFromCoordinates

 Title        : GetNeighboursFromCoordinates
 Usage        : GetNeighboursFromCoordinates($o_entry,$o_project,$o_param)
 Function     : Look for a coordinate file (a light iANT::DB::Entry xml file) and if exists
			 look for neighbours of input entry, then returns a hash with iANT::Entry::Display
			 objects of previous and next entry
 Returns      : $rh_neighbours hash of previous and next iANT::Entry::Dsplay neighbours objects 
 Args         : $o_entry iANT::Entry object
			 $o_project iANT::Project object
			 $o_param
 Globals      : [conf] extension_iant_coordinates

=cut

sub GetNeighboursFromCoordinates
{
    my ($o_entry, $o_project, $o_param) = (@_);

    my $data_dir = $o_project->Get('data_dir');
    $o_entry->SelectLinks({type => "partOf"});
    my $contig_accession = $o_entry->GetLinks->[0]->Get('accession');

    my $extension_iant_coordinates = $o_param->Get('extension_iant_coordinates');

    my $coodinates_file;
    $coodinates_file = "$data_dir/$contig_accession.$extension_iant_coordinates"
      if (-e "$data_dir/$contig_accession.$extension_iant_coordinates");
    $coodinates_file = "$data_dir/$contig_accession/$contig_accession.$extension_iant_coordinates"
      if (-e "$data_dir/$contig_accession/$contig_accession.$extension_iant_coordinates");

    if (defined $coodinates_file)
    {
        my $o_db_entry   = New iANT::DB::Entry($coodinates_file);
        my %h_neighbours = ();
        my $previous     = '';
        foreach my $o_entry_scan (@{$o_db_entry->GetEntries})
        {
            if ($o_entry_scan->GetAccession eq $o_entry->GetAccession)
            {
                $h_neighbours{'previous'} = New iANT::Entry::Display($previous->GetXml);
            }
            elsif (defined $h_neighbours{'previous'})
            {
                $h_neighbours{'next'} = New iANT::Entry::Display($o_entry_scan->GetXml);
                last;
            }
            $previous = $o_entry_scan;

        }
        return \%h_neighbours;
    }
    return 0;
}

=pod

=item function GetNeighboursFromEntry

 Title        : GetNeighboursFromEntry
 Usage        : GetNeighboursFromEntry($o_entry,$o_project,$o_param)
 Function     : look for links partOf , next and previous into input entry;
			 then look for files corresponding to these accessiosn and 
			 returns a hash with iANT::Entry::Display objects of previous and next entry
 Returns      : $rh_neighbours hash of previous and next iANT::Entry::Dsplay neighbours objects 
 Args         : $o_entry iANT::Entry object
			 $o_project iANT::Project object
			 $o_param
 Globals      : [conf] extension_iant_entry

=cut

sub GetNeighboursFromEntry
{
    my ($o_entry, $o_project, $o_param) = (@_);
    my $ra_links = $o_entry->GetLinks;
    my ($contig_accession, $previous_accession, $next_accession) = (undef, undef, undef);

    foreach my $o_link (@{$ra_links})
    {
        if ($o_link->Get('type') eq 'partOf')
        {
            $contig_accession = $o_link->Get('accession');
        }
        elsif ($o_link->Get('type') eq 'previous')
        {
            $previous_accession = $o_link->Get('accession');
        }
        elsif ($o_link->Get('type') eq 'next')
        {
            $next_accession = $o_link->Get('accession');
        }
        last if (defined $contig_accession && defined $previous_accession && defined $next_accession);
    }

    if ((!defined $contig_accession) || (!defined $previous_accession) || (!defined $next_accession))
    {
        return 0;
    }
    else
    {
        my %h_neighbours         = ();
        my $data_dir             = $o_project->Get('data_dir');
        my $extension_iant_entry = $o_param->Get('extension_iant_entry');
        my $previous_entry_file = "$data_dir/$contig_accession/$previous_accession/$previous_accession.$extension_iant_entry";
        $previous_entry_file = "$data_dir/$contig_accession/ft/$previous_accession.$extension_iant_entry"if (! -e $previous_entry_file);
        
        if (-e $previous_entry_file)
        {
            my $o_entry = New iANT::Entry($previous_entry_file);
            $h_neighbours{'previous'} = New iANT::Entry::Display($o_entry->GetXml);
        }

        my $next_entry_file = "$data_dir/$contig_accession/$next_accession/$next_accession.$extension_iant_entry";
        $next_entry_file = "$data_dir/$contig_accession/ft/$next_accession.$extension_iant_entry"if (! -e $next_entry_file);
        if (-e $next_entry_file)
        {
            my $o_entry = New iANT::Entry($next_entry_file);
            $h_neighbours{'next'} = New iANT::Entry::Display($o_entry->GetXml);
        }
        return \%h_neighbours;

    }
    return 0;
    
}

=pod

=item function GetValuesFromParameter

 Title        : GetValuesFromParameter
 Usage        : GetValuesFromParameter($parameter_value)
 Function     : get a parameter value and test if it is a list file, or a list in string form (known separators are [,\s\|;]
 Returns      : list of values reference
 Args         : $parameter_value
 Globals      : none

=cut


sub GetValuesFromParameter
{
    my $parameter_value = shift();
    my @a_values = ();
    
    my $list_separators = '[,\s\|;]+';
    
    if (-e $parameter_value)
    {
        my $fh_values = new IO::File($parameter_value) || Carp::croak("ERROR: $! >$parameter_value<");
        if (defined $fh_values)
        {
            while (my $value = <$fh_values>)
            {
                chomp $value;
                push (@a_values,$value) if ($value ne '');
            }
            $fh_values->close;
        }
    }
    elsif ($parameter_value =~ /$list_separators/ )
    {
        @a_values = split (/$list_separators/,$parameter_value);
    }
    else
    {
       @a_values = ($parameter_value);
    }
    
    return \@a_values;
}

=pod

=item function CleanId

 Title        : CleanId
 Usage        : CleanId($id,$to_remove)
 Function     : remove $to_remove from $id value; 'get_locus_tag' is a particular keyword to get locus-tag from id-locus_tag syntax
 Returns      : cleaned id
 Args         : $id
 		$to_remove
 Globals      : none

=cut

sub CleanId
{
	my ($id,$to_remove) = (@_);
	if ($to_remove eq 'get_locus_tag')
    {
        if ($id =~ /\w+-(\w+)/)
        {
            $id = $1;
        }
    }
    else
    {
	    $id =~ s/$to_remove\|?//g;
    }
	return $id;
}

=pod

=item function RhimeHackSMb

 Title        : RhimeHackSMb
 Usage        : RhimeHackSMb($locus_tag)
 Function     : replaces SMbxxxxx locus_tags into SM_bxxxxx (locus tag typo in German last submission)
 Returns      : coreect locus_tag
 Args         : $locus_tag
 Globals      : none

=cut


sub RhimeHackSMb
{
    my $locus_tag = shift;
    if ($locus_tag =~ /^smb(\d+)/i)
    {
        $locus_tag = 'SM_b' . $1;
    }
    return $locus_tag;

}

sub RalsoHackRS0
{
    my ($locus_tag,$id,$synonym) = @_;
    
    if ($id =~ /^rs[cp]\d+/i)
    {
        $locus_tag = $synonym;
    }
    else
    {
        return "$locus_tag/$synonym";
    }
}

=pod

=head2 LUCENE

=cut

=pod

=head3 function GetLuceneFields4Xml

 Title        : GetLuceneFields4Xml
 Usage        : ($ra_fields_to_parse,$rh_docid) = GetLuceneFields4Xml($o_param)
 Function     : get Fields that have to be parsed (indexed, stored) in lucene
 		from a configuration file for Xml File indexing
 Returns      : $ra_fields_to_parse List reference of fields hash ref (keys: xpath,att,alias,mode); 
 		$rh_docid	hash ref for the docid field (Unique key)
 Args         : $o_param
 Globals      : None

=cut

sub GetLuceneFields4Xml
{
    my $o_param = shift();
    my %h_docid           = ();
    my @a_fields_to_parse = ();

    my @a_field_param = $o_param->GetKeys('field:\S+');
    foreach my $field_param (@a_field_param)
    {
        my ($field_alias) = ($field_param =~ /field:([\w\-]+)/);
        my $field_tmp = $o_param->Get($field_param);

        my ($field_mode, $field_xpath, $field_att);
        if ($field_tmp =~ /\@\w+$/)
        {
            ($field_mode, $field_xpath, $field_att) = ($field_tmp =~ /(\w+)\.(\S+\/?)\@(\S*)/);
        }
        else
        {
            ($field_mode, $field_xpath) = ($field_tmp =~ /(\w+)\.(\S+\/?)/);
        }

        if ($field_alias eq $o_param->Get('docid'))
        {
            %h_docid = (xpath => $field_xpath, att => $field_att, alias => $field_alias, mode => $field_mode);
        }

        if ((!defined $field_xpath) && ($field_tmp =~ /(\w+)\.file/))
        {
            $field_xpath = 'file';
            $field_mode  = $1;
        }

        my %h_field = (xpath => $field_xpath, att => $field_att, alias => $field_alias, mode => $field_mode);
        push(@a_fields_to_parse, \%h_field);

    }

    return (\@a_fields_to_parse, \%h_docid);
}

=pod

=head3 function GetLuceneFields4Txt

 Title        : GetLuceneFields4Txt
 Usage        : ($ra_fields_to_parse,$rh_docid) = GetLuceneFields4Txt($o_param)
 Function     : get Fields that have to be parsed (indexed, stored) in lucene
 		from a configuration file for text File indexing
 Returns      : $ra_fields_to_parse List reference of fields hash ref (keys: regexp,alias,mode); 
 		$rh_docid	hash ref for the docid field (Unique key)
 Args         : $o_param
 Globals      : None

=cut

sub GetLuceneFields4Txt
{
    my $o_param = shift();
    my %h_docid           = ();
    my @a_fields_to_parse = ();

    my @a_field_param = $o_param->GetKeys('field:\S+');
    foreach my $field_param (@a_field_param)
    {
        my ($field_alias) = ($field_param =~ /field:([\w\-]+)/);
        my $field_tmp = $o_param->Get($field_param);

        my ($field_mode, $field_regexp);
        ($field_mode, $field_regexp) = ($field_tmp =~ /(\w+)\.(\S+)/);
        
        if ($field_alias eq $o_param->Get('docid'))
        {
            %h_docid = (regexp => $field_regexp, alias => $field_alias, mode => $field_mode);
        }

        if ((!defined $field_regexp) && ($field_tmp =~ /(\w+)\.file/))
        {
            $field_regexp = 'file';
            $field_mode  = $1;
        }

        my %h_field = (regexp => $field_regexp, alias => $field_alias, mode => $field_mode);
        push(@a_fields_to_parse, \%h_field);

    }

    return (\@a_fields_to_parse, \%h_docid);
}


=pod

=head3 function EscapeLuceneSpecialCharaters

 Title        : EscapeLuceneSpecialCharaters
 Usage        : ($query_value) = EscapeLuceneSpecialCharaters($query_value)
 Function     : backslash (escape) Lucene special characters
		\ + - ! ( ) { } [ ] ^ " ~ * ? 
 Returns      : $query_value escaped value
 Args         : $query_value raw value
 		$ra_no_escape array ref of characters not to escape
 Globals      : None

=cut


sub EscapeLuceneSpecialCharaters
{
	my $query = shift;
	my $no_escape_colon = shift;
	my @a_special_characters = ('\+','-','!','\(','\)','{','}','\[','\]','\^','"','~','\*','\?','_',':');
	if (defined $no_escape_colon)
	{
		pop @a_special_characters;
	}
	
	foreach my $special_character (@a_special_characters)
	{
		$query =~ s/$special_character/\\$special_character/g;
	}
	return $query;
}

#
#AUTHENTICATION
#

sub EncodeHomeMade
{
    my $mdp = shift;
    my $str = Convert::UU::uuencode($mdp);
    $str =~ s/^begin 644 uuencode.uu\s//o;
    $str =~ s/`\send\s$//o;
    $str =~ s/([^A-Za-z0-9])/sprintf("%%%02X", ord($1))/seg;
    $str = reverse($str);
    $str =~ s/%/:/g;
    return $str;
}

sub DecodeHomeMade
{
    my $encoded_substr = shift;
    $encoded_substr =~ s/:/%/g;
    $encoded_substr = reverse($encoded_substr);
    $encoded_substr =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;
    $encoded_substr = "begin 644 uuencode.uu\n$encoded_substr\`\nend\n";

    my $str = Convert::UU::uudecode($encoded_substr);
    return $str;
}




1;
